﻿Type=Class
Version=2.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
'@see readme.txt
'
#Region Notes
'Ensure to call properly from ba.raiseevent else error message like:
'Caused by: java.lang.Exception: Sub UpdateSettings signature does Not match expected signature.
#End Region

#Region Class Attributes 
  	#IgnoreWarnings: 10, 12
#End Region

Sub Class_Globals
	'For debugging
	Private CLASSLOG As Boolean = False
	'Global Map holding the settings
  	Private SettingsMap As Map              
  	'Define the settings file which is read by the global settings map
  	Public SettingsFile As String = "newsreader.set"   
	
	'News folder, file, location and content
	Private NewsFolder As String
	Private NewsFile As String
	Private NewsURL As String
	Private NewsContent As String
	Private NewsList As List
	'XML Structure
	Private ID, Title, Link, Description, PubDate As String
	'XML Parser using SAXParser Library jXmlSax
	Private FXMLParser As SaxParser
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	'Set the folder first before reading the settings
	NewsFolder	= File.DirApp
	'Read the settings first
	ReadSettings
	'Newslist init
	NewsList.Initialize
	'Init the XML parser for reading the news file
  	FXMLParser.Initialize
	'Define the local newsfile and url to download the file from
	NewsFile	= SettingsMap.Get("newsfile") 		'Local filename; also used for the http download job name
  	NewsURL		= SettingsMap.Get("newsurl")   		'Remote filename
	ReadNews
End Sub

Private Sub CLog(s As String)
	If CLASSLOG Then Log(s)
End Sub

Private Sub ReadNews
	DownloadNews
	'joInlineJava.RunMethod("setMessage", Array(NewsContent))
	Log(DateTime.Time(DateTime.Now) & " - News updated.")
End Sub

#Region Settings
'Read the settings = Check if the settings file exists, else create a new file
Public Sub ReadSettings
	CLog("ReadSettings :: Start")
	If File.Exists(NewsFolder, SettingsFile) Then
    	SettingsMap = File.ReadMap(NewsFolder, SettingsFile)
	Else
    	'Create a new map
    	SettingsMap = CreateMap ("interval": "60000", "x":"20", "y":"20","w":"200","h":"100", "stayontop":"1","newsurl":"")
	End If
	CLog("ReadSettings :: End")
End Sub

'Save the settings and store in the setting file located in the dirapps folder
Public Sub SaveSettings(x As Int, y As Int, w As Int, h As Int)
	SettingsMap.Put("x", x)
	SettingsMap.Put("y", y)
	SettingsMap.Put("w", w)
	SettingsMap.Put("h", h)
	File.WriteMap(NewsFolder, SettingsFile, SettingsMap)
End Sub
#End Region

#Region DownloadNews
'Download the newsfile from the internet using httpjob - requires library jHTTPUtils2 
Public Sub DownloadNews
	CLog("DownloadNews :: Start")
	'HTTP job for download
	Dim job1 As HttpJob
	'Clear all items first
	NewsList.Clear
	'Delete the newsfile (if exists)
	If File.Exists(NewsFolder, NewsFile) Then File.Delete(NewsFolder, NewsFile)
	'Init the download job using the newsfile as the Jobname
  	Try
		job1.Initialize(NewsFile, Me)
		job1.Download(NewsURL)
    Catch
	  	NewsList.Add("Error Init download: " & LastException.Message)
      	CLog("Job Init Error: " & LastException.Message)
    End Try	
  	'Use JobDone Event to action result --- see below
	CLog("DownloadNews :: End")
End Sub

'The jobname is used as the filename to store the local in the app folder (same as the jar file)
'First check that Job.Success is true and only then access the response.
Private Sub JobDone (Job As HttpJob)
	CLog("JobDone :: Start")
 	CLog("Job result for JobName = " & Job.JobName & ", Success = " & Job.Success)
  	'Job done ok, lets save the content to the local file
  	If Job.Success = True Then                          
    	Try
      		Dim out As OutputStream = File.OpenOutput(NewsFolder, Job.JobName, False)
      		File.Copy2(Job.GetInputStream, out)
			out.Close
      		CLog("Job Done. Local file created: " & Job.JobName & " (" & NewsFolder & ")")
	  		ParseNews
    	Catch
	  		NewsList.Add("Error News download: " & LastException.Message)
      		CLog("Job Error. Can not create local file: " & LastException.Message)
    	End Try
  	Else
	  	NewsList.Add("<H1>Error</H1><P>News not downloaded. Message from Server: </P><P>" & Job.ErrorMessage & "</P>")
    	CLog("Job Error. Can not download file: " & Job.ErrorMessage)
  	End If
  	Job.Release
	CLog("Build NewsContent")
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To NewsList.Size - 1
		sb.Append(NewsList.get(i))
	Next
	NewsContent = sb.tostring
	Log(NewsContent)
	CLog("JobDone :: End")
	'And exit the app
	ExitApplication
End Sub
#End Region

#Region ParseNews
'XML Structure
'  <item id="3">
'		<title>B4J HowTo News Reader Framework</title>
'		<link>http://www.rwblinn.de</link>
'		<description>Started To develop a News Reader Framework</description>
'		<pubdate>20150517 13:59</pubdate>
'  </item>

Private Sub ParseNews
	CLog("ParseNews :: Start")
	'Check first if news file exist
	If File.Exists(NewsFolder, NewsFile) = False Then
		NewsList.Add("Error News file not found.")
    	CLog("Error: XML file to parse not found.")
    	Return
  	End If
	'parse the xml file
  	Dim In As InputStream
  	Try
    	In = File.OpenInput(NewsFolder, NewsFile)
    	FXMLParser.Parse(In, "Parser")
    	In.Close
		'
		'Webview LoadHTML file
		'webviewNews.LoadHTML(NewsContent)
		'SetStatusMessage(DateTime.Time(DateTime.Now) & " - News updated.")
    	CLog("Information: Imported News from XML File: " & NewsFile)
 	Catch
	  	NewsList.Add("Error News file not parsed.")
    	CLog("Error: XML Import: " & LastException.Message)
  	End Try
	CLog("ParseNews :: End")
End Sub

'XML parser startelement
'Read the Item and get the Attribute ID
Sub Parser_StartElement (Uri As String, Name As String, Attributes As Attributes)
	'Start item element found, get the attribute id <item id="1">
	If Name = "item" Then
		For i = 0 To Attributes.Size -1
			If Attributes.GetName(i).ToLowerCase = "id" Then ID = Attributes.GetValue(i)
			'Log(Attributes.GetName(i) & " = " & Attributes.GetValue(i))
		Next
		'Reset the vars - do not forget
		Title = ""
		Link = ""
		PubDate = ""
		Description = ""
	End If
End Sub

Sub Parser_EndElement (Uri As String, Name As String, Text As StringBuilder)
	If FXMLParser.Parents.IndexOf("item") > -1 Then
		If Name = "title" Then
			Title = Text.ToString
		Else If Name = "link" Then
			Link = Text.ToString
		Else If Name = "pubdate" Then
			PubDate = Text.ToString
		Else If Name = "description" Then
			Description = Text.ToString
		Else If Name = "ID" Then
			ID = Text.ToString
		End If
	End If
	'End Element Found </item>
	If Name = "item" Then
		'Add the item: UI = B4A + B4JUI, NONUIX = Console + Java Swing, NONUI = Console
		NewsList.Add(Title & CRLF & PubDate & Description & CRLF & Link & CRLF)
	End If
End Sub
#End Region
