﻿Type=Class
Version=2.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
'@see readme.txt
'
#Region Notes
'Ensure to call properly from ba.raiseevent else error message like:
'Caused by: java.lang.Exception: Sub UpdateSettings signature does Not match expected signature.
#End Region

#Region Class Attributes 
  	#IgnoreWarnings: 10, 12
#End Region

Sub Class_Globals
	'For debugging
	Private CLASSLOG As Boolean = False
	'Define the javaobject to call the methods from the inline java section
	Public joInlineJava As JavaObject = Me
	'
	'Global Map holding the settings
  	Private SettingsMap As Map              
  	'Define the settings file which is read by the global settings map
  	Public SettingsFile As String = "newsreader.set"   
	
	'News folder, file, location and content
	Private NewsFolder As String
	Private NewsFile As String
	Private NewsURL As String
	Private NewsContent As String
	Private NewsList As List
	'XML Structure
	Private ID, Title, Link, Description, PubDate As String
	'XML Parser using SAXParser Library jXmlSax
	Private FXMLParser As SaxParser
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	'Set the folder first before reading the settings
	NewsFolder	= File.DirApp
	'Read the settings first
	ReadSettings
	'Newslist init
	NewsList.Initialize
	'Init the XML parser for reading the news file
  	FXMLParser.Initialize
	'Define the local newsfile and url to download the file from
	NewsFile	= SettingsMap.Get("newsfile") 		'Local filename; also used for the http download job name
  	NewsURL		= SettingsMap.Get("newsurl")   		'Remote filename
	'Initialize(webviewNews, File.dirapp)
	'Create the widgetframe by calling the inline java method setWidgetUI
	Dim x As Int = SettingsMap.Get("x")
	Dim y As Int = SettingsMap.Get("y")
	Dim w As Int = SettingsMap.Get("w")
	Dim h As Int = SettingsMap.Get("h")
	Dim t As Boolean = (SettingsMap.Get("stayontop") = "1")
	'
	joInlineJava.RunMethod("setWidgetUI", Array(x,y,w,h,t))
	joInlineJava.RunMethod("setStatusMessage", Array("Welcome - Select a Widget"))
	'Get the latest news
	joInlineJava.RunMethod("setMessage", Array("Please wait, getting the latest News ..."))
	ReadNews
End Sub

Private Sub CLog(s As String)
	If CLASSLOG Then Log(s)
End Sub

Private Sub ReadNews
	DownloadNews
	'joInlineJava.RunMethod("setMessage", Array(NewsContent))
	joInlineJava.RunMethod("setStatusMessage", Array(DateTime.Time(DateTime.Now) & " - News updated."))
End Sub

#Region Settings
'Read the settings = Check if the settings file exists, else create a new file
Public Sub ReadSettings
	CLog("ReadSettings :: Start")
	If File.Exists(NewsFolder, SettingsFile) Then
    	SettingsMap = File.ReadMap(NewsFolder, SettingsFile)
	Else
    	'Create a new map
    	SettingsMap = CreateMap ("interval": "60000", "x":"20", "y":"20","w":"200","h":"100", "stayontop":"1","newsurl":"")
	End If
	CLog("ReadSettings :: End")
End Sub

'Save the settings and store in the setting file located in the dirapps folder
Public Sub SaveSettings(x As Int, y As Int, w As Int, h As Int)
	SettingsMap.Put("x", x)
	SettingsMap.Put("y", y)
	SettingsMap.Put("w", w)
	SettingsMap.Put("h", h)
	File.WriteMap(NewsFolder, SettingsFile, SettingsMap)
End Sub
#End Region

#Region DownloadNews
'Download the newsfile from the internet using httpjob - requires library jHTTPUtils2 
Public Sub DownloadNews
	CLog("DownloadNews :: Start")
	'HTTP job for download
	Dim job1 As HttpJob
	'Clear all items first
	NewsList.Clear
	'Delete the newsfile (if exists)
	If File.Exists(NewsFolder, NewsFile) Then File.Delete(NewsFolder, NewsFile)
	'Init the download job using the newsfile as the Jobname
  	Try
		job1.Initialize(NewsFile, Me)
		job1.Download(NewsURL)
    Catch
	  	NewsList.Add("Error Init download: " & LastException.Message)
      	CLog("Job Init Error: " & LastException.Message)
    End Try	
  	'Use JobDone Event to action result --- see below
	CLog("DownloadNews :: End")
End Sub

'The jobname is used as the filename to store the local in the app folder (same as the jar file)
'First check that Job.Success is true and only then access the response.
Private Sub JobDone (Job As HttpJob)
	CLog("JobDone :: Start")
 	CLog("Job result for JobName = " & Job.JobName & ", Success = " & Job.Success)
  	'Job done ok, lets save the content to the local file
  	If Job.Success = True Then                          
    	Try
      		Dim out As OutputStream = File.OpenOutput(NewsFolder, Job.JobName, False)
      		File.Copy2(Job.GetInputStream, out)
			out.Close
      		CLog("Job Done. Local file created: " & Job.JobName & " (" & NewsFolder & ")")
	  		ParseNews
    	Catch
	  		NewsList.Add("Error News download: " & LastException.Message)
      		CLog("Job Error. Can not create local file: " & LastException.Message)
    	End Try
  	Else
	  	NewsList.Add("<H1>Error</H1><P>News not downloaded. Message from Server: </P><P>" & Job.ErrorMessage & "</P>")
    	CLog("Job Error. Can not download file: " & Job.ErrorMessage)
  	End If
  	Job.Release
	CLog("Build NewsContent")
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To NewsList.Size - 1
		sb.Append(NewsList.get(i))
	Next
	NewsContent = sb.tostring
	Main.NewsReader.joInlineJava.RunMethod("setMessage", Array(NewsContent))
	CLog("JobDone :: End")
End Sub
#End Region

#Region ParseNews
'XML Structure
'  <item id="3">
'		<title>B4J HowTo News Reader Framework</title>
'		<link>http://www.rwblinn.de</link>
'		<description>Started To develop a News Reader Framework</description>
'		<pubdate>20150517 13:59</pubdate>
'  </item>

Private Sub ParseNews
	CLog("ParseNews :: Start")
	'Check first if news file exist
	If File.Exists(NewsFolder, NewsFile) = False Then
		NewsList.Add("Error News file not found.")
    	CLog("Error: XML file to parse not found.")
    	Return
  	End If
	'parse the xml file
  	Dim In As InputStream
  	Try
    	In = File.OpenInput(NewsFolder, NewsFile)
    	FXMLParser.Parse(In, "Parser")
    	In.Close
		'
		'Webview LoadHTML file
		'webviewNews.LoadHTML(NewsContent)
		'SetStatusMessage(DateTime.Time(DateTime.Now) & " - News updated.")
    	CLog("Information: Imported News from XML File: " & NewsFile)
 	Catch
	  	NewsList.Add("Error News file not parsed.")
    	CLog("Error: XML Import: " & LastException.Message)
  	End Try
	CLog("ParseNews :: End")
End Sub

'XML parser startelement
'Read the Item and get the Attribute ID
Sub Parser_StartElement (Uri As String, Name As String, Attributes As Attributes)
	'Start item element found, get the attribute id <item id="1">
	If Name = "item" Then
		For i = 0 To Attributes.Size -1
			If Attributes.GetName(i).ToLowerCase = "id" Then ID = Attributes.GetValue(i)
			'Log(Attributes.GetName(i) & " = " & Attributes.GetValue(i))
		Next
		'Reset the vars - do not forget
		Title = ""
		Link = ""
		PubDate = ""
		Description = ""
	End If
End Sub

Sub Parser_EndElement (Uri As String, Name As String, Text As StringBuilder)
	If FXMLParser.Parents.IndexOf("item") > -1 Then
		If Name = "title" Then
			Title = Text.ToString
		Else If Name = "link" Then
			Link = Text.ToString
		Else If Name = "pubdate" Then
			PubDate = Text.ToString
		Else If Name = "description" Then
			Description = Text.ToString
		Else If Name = "ID" Then
			ID = Text.ToString
		End If
	End If
	'End Element Found </item>
	If Name = "item" Then
		'Add the item: UI = B4A + B4JUI, NONUIX = Console + Java Swing, NONUI = Console
		NewsList.Add("<div id=""" & ID & """><B>" & Title & "</B><BR>" & PubDate & "&nbsp;" & Description & "<BR><A HREF=""" & Link & """>" & Link & "</A><HR>")
	End If
End Sub
#End Region

#If JAVA

/*
 * import anywheresoftware.b4a.keywords.Common;
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

//Create the frame
//Define Public because updated from B4J Sub
public final JFrame frame = new JFrame("News Reader");

//Define the label which shows the status after sending an email
public static JLabel labelStatus = new JLabel("", JLabel.CENTER);

public static JButton buttonChecked = new JButton("Checked");

//Define the label showing the Message As Public because updated from B4J Sub
public static JEditorPane edpaneMessage = new JEditorPane();

//Create the widget UI which Is a jframe composed out of controls To, Subject, Body, Button Send, Message
//IMPORTANT: use lowercase For raising events!
//REMOVE Status
public void setWidgetUI(int x, int y, int w, int h, boolean stayontop) {
	//Set frame Type To utility
	frame.setType(javax.swing.JFrame.Type.UTILITY);
	frame.setAlwaysOnTop (stayontop);
	//Set frame type to undecorated as another option --- NOT USED
	//frame.setUndecorated(True);
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	//Add listener To the Title bar close button top right [x]
	frame.addWindowListener(new WindowAdapter(){
		public void windowClosing(WindowEvent e){
			//System.out.println("updatesettings/" + frame.getX() + "/" + frame.getY() + "/" + frame.getWidth() + "/" + frame.getHeight());
		    ba.raiseEvent(this, "updatesettings", frame.getX(), frame.getY(), frame.getWidth(), frame.getHeight());
			System.exit(0);
		}
	});

	JPanel vPanel = new JPanel();
	vPanel.setLayout(new BoxLayout(vPanel, BoxLayout.Y_AXIS));

	//EditorPane
	//Set the context Type Text/html And uneditable For links To be clickable
	edpaneMessage.setEditorKit(JEditorPane.createEditorKitForContentType("text/html"));
	edpaneMessage.setEditable(false);    
	
	//Add mouse listener To the editorpane
	//double click exists the ExitApplication. This can used For a rasperry pi when the app has undecorated style (right mouse click)
	edpaneMessage.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
			    ba.raiseEvent(this, "updatesettings", frame.getX(), frame.getY(), frame.getWidth(), frame.getHeight());
				System.exit(0);
			}
        }
    });

	//Add a hyperlink listener
	edpaneMessage.addHyperlinkListener(new HyperlinkListener() {
		@Override
		public void hyperlinkUpdate(HyperlinkEvent e) {
			//System.out.println(e.getURL());
			if (e.getEventType()==HyperlinkEvent.EventType.ACTIVATED) {
				//System.out.println(e.getSource().toString());
				if(Desktop.isDesktopSupported()) {
					try {
                		Desktop.getDesktop().browse(e.getURL().toURI());
            		} 
					catch (Exception e1) {
                    	e1.printStackTrace();
                	}
				}			
			}
		}
	});
	
	//Add the controls To the gridpane	
	edpaneMessage.setToolTipText("Double Click to close the Widget");
	JScrollPane scrollPane = new JScrollPane(edpaneMessage);
	vPanel.add(scrollPane, BorderLayout.CENTER);
			
    JPanel southPanel = new JPanel();
    southPanel.setLayout(new GridLayout(0,1));
	//Button Refresh
    JButton buttonRefresh = new JButton("Refresh");
	//buttonRefresh.setBackground(Color.BLUE);
	//buttonRefresh.setForeground(Color.WHITE);
	buttonRefresh.setToolTipText("Click to refresh the news.");
	buttonRefresh.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
			ba.raiseEvent(this, "readnews", null);
		}
	});
    southPanel.add(buttonRefresh, BorderLayout.SOUTH);
	southPanel.setMaximumSize( southPanel.getPreferredSize() );
	vPanel.add(southPanel, BorderLayout.SOUTH);
    frame.getContentPane().add(vPanel);
    //
    labelStatus.setToolTipText("Double Click to close the Widget.");
	//Add mouse listener To the statuslabel
	//double click exists the ExitApplication. This can used For a rasperry pi when the app has undecorated style (right mouse click)
	labelStatus.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
			    ba.raiseEvent(this, "updatesettings", frame.getX(), frame.getY(), frame.getWidth(), frame.getHeight());
				System.exit(0);
			}
        }
   	});
    frame.getContentPane().add(labelStatus, BorderLayout.SOUTH);
  	//Set the location
	//frame.setLocationRelativeTo(Null); Not USED
	frame.setLocation(x, y);
	frame.setSize(w, h);
	//frame.pack();
	frame.setVisible(true);
}	

//Set the message editor pane showing the HTML formatted message
public static void setMessage(String s) {
	edpaneMessage.setText(s);
}

//Update the status label at the bottom of the UI
public static void setStatusMessage(String message){
	labelStatus.setText(message);
}

//Bring up a dialog to show the message FROM and SUBJECT
public static void MessageDialog(JFrame frame, String str) {
	if(str != null && !str.isEmpty()) {
		JOptionPane.showMessageDialog(frame, str);
	}
};

/*
 * End If JAVA
 */

#End If
